package com.io.yutian.refinesystem;

import com.io.yutian.refinesystem.data.RefineData;
import com.io.yutian.refinesystem.data.SkillData;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;

import java.util.HashMap;
import java.util.Map;

public class RefineSystemSetting {

    private static String detectionLore;
    private static Map<String, RefineData> refineDatas = new HashMap<>();
    private static Map<String, Integer> tierLevels = new HashMap<>();

    public static void reload() {
        refineDatas.clear();
        RefineSystem.inst().saveDefaultConfig();
        RefineSystem.inst().reloadConfig();
        FileConfiguration config = RefineSystem.inst().getConfig();
        detectionLore = config.getString("detectionLore");
        ConfigurationSection section = config.getConfigurationSection("options");
        for (String key : section.getKeys(false)) {
            double chance = section.getDouble(key+".chance");
            int needQuality = section.getInt(key+".needQuality");
            int minGlossaryAmount = section.getInt(key+".glossaryAmount.min");
            int maxGlossaryAmount = section.getInt(key+".glossaryAmount.max");
            ConfigurationSection skillSection = section.getConfigurationSection(key+".skills");
            Map<String, SkillData> skillDatas = new HashMap<>();
            for (String skillKey : skillSection.getKeys(false)) {
                double skillChance = skillSection.getDouble(skillKey+".chance");
                int minSkillLevel = skillSection.getInt(skillKey+".level.min");
                int maxSkillLevel = skillSection.getInt(skillKey+".level.max");
                double minTrigger = skillSection.getDouble(skillKey+".trigger.min");
                double maxTrigger = skillSection.getDouble(skillKey+".trigger.max");
                skillDatas.put(skillKey, new SkillData(skillKey, skillChance, minSkillLevel, maxSkillLevel, minTrigger, maxTrigger));
            }
            refineDatas.put(key, new RefineData(key, chance, needQuality, minGlossaryAmount, maxGlossaryAmount, skillDatas));
        }
        ConfigurationSection tierSection = config.getConfigurationSection("tierLevels");
        for (String tierKey : tierSection.getKeys(false)) {
            int tierLevel = tierSection.getInt(tierKey);
            tierLevels.put(tierKey, tierLevel);
        }
    }

    public static String getDetectionLore() {
        return detectionLore;
    }

    public static boolean hasRefineData(String id) {
        return refineDatas.containsKey(id);
    }

    public static RefineData getRefineData(String id) {
        return refineDatas.get(id);
    }

    public static Map<String, RefineData> getRefineDatas() {
        return refineDatas;
    }

    public static Map<String, Integer> getTierLevels() {
        return tierLevels;
    }

}
