package com.io.yutian.refinesystem.gui;

import com.io.yutian.refinesystem.RefineSystemSetting;
import com.io.yutian.refinesystem.data.RefineData;
import com.io.yutian.refinesystem.data.SkillData;
import com.io.yutian.refinesystem.util.ItemStackUtil;
import com.io.yutian.refinesystem.util.PlayerInventoryUtil;
import com.io.yutian.refinesystem.util.RandomBuilder;
import com.io.yutian.refinesystem.util.RandomUtil;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.Sound;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.inventory.InventoryCloseEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.InventoryHolder;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class RefineGui implements InventoryHolder, GuiHolder {

    private static final int[] ARRAY_1 = new int[] {0,2,3,5,6,8,12,14,18,20,21,23,24,26};
    private static final int[] ARRAY_2 = new int[] {1,7,9,11,15,17,19,25};
    private static final int[] ARRAY_3 = new int[] {4,22};

    private static final ItemStack ITEM_1;
    private static final ItemStack ITEM_2;
    private static final ItemStack ITEM_3;
    private static final ItemStack ITEM_4;
    private static final ItemStack ITEM_5;
    private static final ItemStack ITEM_6;
    private static final ItemStack ITEM_7;
    private static final ItemStack ITEM_10;

    private Inventory inventory;
    private Player player;

    private ItemStack inputItem;
    private ItemStack refineItem;

    public RefineGui(Player player) {
        this.player = player;
        this.inventory = Bukkit.createInventory(this, 27, "洗练系统");
        for (int i : ARRAY_1) {
            this.inventory.setItem(i, ITEM_1);
        }
        for (int i : ARRAY_2) {
            this.inventory.setItem(i, ITEM_2);
        }
        for (int i : ARRAY_3) {
            this.inventory.setItem(i, ITEM_3);
        }
        this.inventory.setItem(10, ITEM_4);
        this.inventory.setItem(13, ITEM_10);
        this.inventory.setItem(16, ITEM_6);
    }

    @Override
    public void onClick(InventoryClickEvent event) {
        Inventory clickedInventory = event.getClickedInventory();
        int slot = event.getRawSlot();
        event.setCancelled(true);
        if (slot < 27) {
            if (slot == 10) {
                removeInputItem(10);
            } else if (slot == 16) {
                removeRefineItem(16);
            } else if (slot == 13) {
                refine();
            }
        } else {
            ItemStack clickedItem = event.getCurrentItem();
            if (clickedItem == null || clickedItem.getType() == Material.AIR) {
                return;
            }
            if (ItemStackUtil.isRefineItem(clickedItem)) {
                setRefineItem(clickedItem, clickedInventory, event.getSlot());
                return;
            }
            if (ItemStackUtil.canRefine(clickedItem)) {
                setInputItem(clickedItem, clickedInventory, event.getSlot());
                return;
            }
            if (this.inputItem == null || this.refineItem == null) {
                player.sendMessage("§8§l[§e§l!§8§l] §7该物品不是可洗练的物品或洗练符");
                player.playSound(player.getLocation(), Sound.ENTITY_VILLAGER_NO, 1, 1.6F);
            }
        }
    }

    public void setInputItem(ItemStack inputItem, Inventory inventory, int slot) {
        if (this.inputItem != null) {
            return;
        }
        this.inputItem = inputItem;
        update();
        inventory.setItem(slot, null);
        player.playSound(player.getLocation(), Sound.UI_BUTTON_CLICK, 1, 1);
    }

    public void setRefineItem(ItemStack refineItem, Inventory inventory, int slot) {
        if (this.refineItem != null) {
            return;
        }
        int amount = refineItem.getAmount();
        ItemStack itemStack1 = refineItem.clone();
        ItemStack itemStack2 = refineItem.clone();
        if (amount > 1) {
            itemStack1.setAmount(1);
            itemStack2.setAmount(amount - 1);
        } else {
            itemStack2 = null;
        }
        this.refineItem = itemStack1;
        update();
        inventory.setItem(slot, itemStack2);
        player.playSound(player.getLocation(), Sound.UI_BUTTON_CLICK, 1, 1);
    }

    public void removeInputItem(int index) {
        if (this.inputItem == null) {
            return;
        }
        if (!PlayerInventoryUtil.hasFreeSlot(player)) {
            player.sendMessage("§8§l[§c§l!§8§l] §7您的背包空间不足,无法拿下物品");
            return;
        }
        PlayerInventoryUtil.giveItemStack(player, this.inputItem);
        this.inputItem = null;
        update();
        player.playSound(player.getLocation(), Sound.UI_BUTTON_CLICK, 1, 1);
    }

    public void removeRefineItem(int index) {
        if (this.refineItem == null) {
            return;
        }
        if (!PlayerInventoryUtil.hasFreeSlot(player)) {
            player.sendMessage("§8§l[§c§l!§8§l] §7您的背包空间不足,无法拿下物品");
            return;
        }
        PlayerInventoryUtil.giveItemStack(player, this.refineItem);
        this.refineItem = null;
        update();
        player.playSound(player.getLocation(), Sound.UI_BUTTON_CLICK, 1, 1);
    }

    public void refine() {
        if (this.inputItem == null || this.refineItem == null) {
            return;
        }
        int tierLevel = ItemStackUtil.getTierLevel(this.inputItem);
        String refineType = ItemStackUtil.getRefineType(this.refineItem);
        if (!RefineSystemSetting.hasRefineData(refineType)) {
            return;
        }
        RefineData refineData = RefineSystemSetting.getRefineData(refineType);
        if (tierLevel < refineData.getNeedQuality()) {
            player.sendMessage("§8§l[§c§l!§8§l] §7该物品的品质未达到洗练要求,无法洗练");
            player.playSound(player.getLocation(), Sound.ENTITY_VILLAGER_NO, 1, 1.6F);
            return;
        }
        boolean success = RandomUtil.random(refineData.getChance());
        ItemStack itemStack = this.inputItem;
        if (success) {
            player.playSound(player.getLocation(), Sound.ENTITY_PLAYER_LEVELUP, 1, 1);
            player.sendMessage("§8§l[§a§l!§8§l] §7恭喜您,洗炼成功!");
            itemStack = applyRefine(this.inputItem, refineData);
        } else {
            player.playSound(player.getLocation(), Sound.ENTITY_BLAZE_DEATH, 1, 1);
            player.sendMessage("§8§l[§e§l!§8§l] §7很遗憾,洗炼失败");
        }
        this.inputItem = null;
        this.refineItem = null;
        new RefineResultGui(player, itemStack).open();
    }

    public ItemStack applyRefine(ItemStack inputItem, RefineData refineData) {
        ItemStack itemStack = inputItem.clone();
        int glossaryAmount = RandomUtil.getRandomInt(refineData.getMinGlossaryAmount(), refineData.getMaxGlossaryAmount());
        List<String> lores = new ArrayList<>();
        Map<String, Double> cacheMap = new HashMap<>(refineData.getSkillChances());
        for (int i = 0; i < glossaryAmount; i++) {
            RandomBuilder<String> randomBuilder = new RandomBuilder<>(cacheMap);
            String skill = randomBuilder.random();
            if (skill == null) {
                continue;
            }
            cacheMap.remove(skill);
            SkillData skillData = refineData.getSkillDatas().get(skill);
            lores.add(skillData.randomSkillLore());
        }
        itemStack = ItemStackUtil.replaceLore(itemStack, RefineSystemSetting.getDetectionLore(), lores);
        return itemStack;
    }

    private void update() {
        this.inventory.setItem(10, this.inputItem != null ? this.inputItem : ITEM_4);
        this.inventory.setItem(16, this.refineItem != null ? this.refineItem : ITEM_6);
    }

    @Override
    public void onClose(InventoryCloseEvent event) {
        if (this.inputItem != null) {
            PlayerInventoryUtil.giveItemStack(player, this.inputItem);
        }
        if (this.refineItem != null) {
            PlayerInventoryUtil.giveItemStack(player, this.refineItem);
        }
        this.inputItem = null;
        this.refineItem = null;
    }

    @Override
    public void open() {
        player.openInventory(inventory);
    }

    @Override
    public Inventory getInventory() {
        return inventory;
    }

    static {
        ITEM_1 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 15);
        ITEM_2 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 0);
        ITEM_3 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 5);
        ITEM_4 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 14);
        ITEM_5 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 1);
        ITEM_6 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 2);
        ITEM_7 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 14);
        ITEM_10 = new ItemStack(Material.ANVIL, 1);
        ItemMeta itemMeta1 = ITEM_1.getItemMeta();
        itemMeta1.setDisplayName(" ");
        ITEM_1.setItemMeta(itemMeta1);
        ItemMeta itemMeta2 = ITEM_2.getItemMeta();
        itemMeta2.setDisplayName("");
        ITEM_2.setItemMeta(itemMeta2);
        ItemMeta itemMeta3 = ITEM_3.getItemMeta();
        itemMeta3.setDisplayName(" ");
        ITEM_3.setItemMeta(itemMeta3);
        ItemMeta itemMeta4 = ITEM_4.getItemMeta();
        itemMeta4.setDisplayName("§a物品孔");
        ITEM_4.setItemMeta(itemMeta4);
        ItemMeta itemMeta5 = ITEM_5.getItemMeta();
        itemMeta5.setDisplayName(" ");
        ITEM_5.setItemMeta(itemMeta5);
        ItemMeta itemMeta6 = ITEM_6.getItemMeta();
        itemMeta6.setDisplayName("§a洗练符孔");
        ITEM_6.setItemMeta(itemMeta6);
        ItemMeta itemMeta7 = ITEM_7.getItemMeta();
        itemMeta7.setDisplayName(" ");
        ITEM_7.setItemMeta(itemMeta7);
        ItemMeta itemMeta10 = ITEM_10.getItemMeta();
        itemMeta10.setDisplayName("§f点击洗练");
        ITEM_10.setItemMeta(itemMeta10);
    }

}
