package com.io.yutian.refinesystem.util;

import com.io.yutian.refinesystem.RefineSystemSetting;
import de.tr7zw.itemnbtapi.NBTItem;
import org.bukkit.Material;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class ItemStackUtil {

    public static boolean canRefine(ItemStack itemStack) {
        if (itemStack == null || itemStack.getType() == Material.AIR) {
            return false;
        }
        ItemMeta itemMeta = itemStack.getItemMeta();
        if (!itemMeta.hasLore()) {
            return false;
        }
        List<String> lore = itemMeta.getLore();
        for (String s : lore) {
            if (s.contains(RefineSystemSetting.getDetectionLore())) {
                return true;
            }
        }
        return false;
    }

    public static boolean isRefineItem(ItemStack itemStack) {
        if (itemStack == null || itemStack.getType() == Material.AIR) {
            return false;
        }
        NBTItem nbtItem = new NBTItem(itemStack);
        if (nbtItem.hasKey("refineType")) {
            return true;
        }
        return false;
    }

    public static boolean isRemoveRefineItem(ItemStack itemStack) {
        if (itemStack == null || itemStack.getType() == Material.AIR) {
            return false;
        }
        NBTItem nbtItem = new NBTItem(itemStack);
        if (nbtItem.hasKey("removeRefine")) {
            return true;
        }
        return false;
    }

    public static String getRefineType(ItemStack itemStack) {
        if (itemStack == null || itemStack.getType() == Material.AIR) {
            return null;
        }
        NBTItem nbtItem = new NBTItem(itemStack);
        if (nbtItem.hasKey("refineType")) {
            return nbtItem.getString("refineType");
        }
        return null;
    }

    public static int getTierLevel(ItemStack itemStack) {
        if (itemStack == null || itemStack.getType() == Material.AIR) {
            return -1;
        }
        ItemMeta itemMeta = itemStack.getItemMeta();
        if (!itemMeta.hasLore()) {
            return -1;
        }
        List<String> lore = itemMeta.getLore();
        for (String s : lore) {
            for (Map.Entry<String, Integer> entry : RefineSystemSetting.getTierLevels().entrySet()) {
                if (s.contains(entry.getKey())) {
                    return entry.getValue();
                }
            }
        }
        return -1;
    }

    public static boolean isRefined(ItemStack itemStack) {
        if (itemStack == null || itemStack.getType() == Material.AIR) {
            return false;
        }
        ItemMeta itemMeta = itemStack.getItemMeta();
        if (!itemMeta.hasLore()) {
            return false;
        }
        List<String> lore = itemMeta.getLore();
        for (String s : lore) {
            if (s.contains("§7(被动技)")) {
                return true;
            }
        }
        return false;
    }

    public static ItemStack replaceAllLore(ItemStack itemStack, String newLore) {
        if (itemStack == null || itemStack.getType() == Material.AIR) {
            return itemStack;
        }
        ItemMeta itemMeta = itemStack.getItemMeta();
        if (!itemMeta.hasLore()) {
            return itemStack;
        }
        List<String> lore = itemMeta.getLore();
        int index = -1;
        for (int i = 0; i < lore.size(); i++) {
            String s = lore.get(i);
            if (s.contains("§7(被动技)")) {
                if (index == -1) {
                    index = i;
                }
            }
        }
        lore.set(index,newLore);
        Iterator<String> iterator = lore.iterator();
        while (iterator.hasNext()) {
            String s = iterator.next();
            if (s.contains("§7(被动技)")) {
                iterator.remove();
            }
        }
        itemMeta.setLore(lore);
        itemStack.setItemMeta(itemMeta);
        return itemStack;
    }

    public static ItemStack replaceLore(ItemStack itemStack, String oldLore, List<String> newLores) {
        ItemStack itemStack1 = itemStack.clone();
        ItemMeta itemMeta = itemStack1.getItemMeta();
        List<String> lores = itemMeta.hasLore() ? itemMeta.getLore() : new ArrayList<>();
        int index = -1;
        for (int i = 0; i < lores.size(); i++) {
            String s = lores.get(i);
            if (s.contains(oldLore)) {
                index = i;
                break;
            }
        }
        lores.remove(index);
        int i = 0;
        for (String s : newLores) {
            lores.add(index+i, s);
            i++;
        }
        itemMeta.setLore(lores);
        itemStack1.setItemMeta(itemMeta);
        return itemStack1;
    }

}
